<?php

namespace Drupal\social_event_invite_flow\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\virtual_event_bbb\VirtualEventBBB;
use BigBlueButton\Parameters\JoinMeetingParameters;
use Drupal\Core\Url;
use Drupal\Core\Routing\TrustedRedirectResponse;

/**
 * Provides a Social event invite flow form.
 */
class JoinVirtualEventAsGuest extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'social_event_invite_flow_join_virtual_event_as_guest';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $nid = NULL) {

    // First let's check if we have an existing enrollment for
    // the current user.
    $event_invite_flow_service = \Drupal::service('social_event_invite_flow.invite_flow_service');
    $sid = $event_invite_flow_service->getSessionIDForUnauthenticatedUsers();

    // Check if we already have an enrolled anon. user
    $enrollment = $event_invite_flow_service->enrollmentExistsBySID($sid, $nid);

    $form['#attributes'] = [
      'class' => ['visually-hidden']
    ];

    $form['fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Join Meeting now'),
      '#collapsed' => FALSE,
    ];


    $form['fieldset']['event_id'] = [
      '#type' => 'hidden',
      '#value' => $nid,
    ]; 
    
    if ($enrollment) {
      $form['fieldset']['name'] = [
        '#type' => 'html_tag',
        '#tag' => 'h3',
        '#value' => $this->t('Hello @user!',['@user' => $enrollment->field_first_name->value]),
      ];
    }
    else {
      $form['fieldset']['name'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Name')
      ];
    }

    
    $form['fieldset']['actions'] = [
      '#type' => 'actions',
    ];

    $form['fieldset']['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('JOIN MEETING'),
      '#attributes' => [
        'class' => [
          'btn-accent',
          'button',
          'btn',
          'btn-lg',
          'waves-effect',
          'waves-btn',
          'waves-light',
        ]
      ]
    ];

    /** @var \Drupal\data_policy\DataPolicyConsentManagerInterface $data_policy_manager */
    $data_policy_manager = \Drupal::service('data_policy.manager');

    if (!$data_policy_manager->isDataPolicy()) {
      return;
    }
    
    $data_policy_manager->addCheckbox($form);

    return $form;
  
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    // Get the event id (node)
    $event_id = $form_state->getValue('event_id');

    // Get the name from the form
    $name = $form_state->getValue('name');    

    // First let's check if we have an existing enrollment for
    // the current user.
    $event_invite_flow_service = \Drupal::service('social_event_invite_flow.invite_flow_service');
    $sid = $event_invite_flow_service->getSessionIDForUnauthenticatedUsers();
    $enrollment = $event_invite_flow_service->enrollmentExistsBySID($sid, $event_id);

    if ($enrollment) {
      $name = $enrollment->field_first_name->value;      
    }
    else {
      // Enroll guest when not enrolled
      $event_invite_flow_service->enrollGuestInEvent($event_id, $name);
    }

    if (isset($name)) {
      $node = \Drupal::entityTypeManager()->getStorage('node')->load($event_id);
    
      $BBBKeyPluginManager = \Drupal::service('plugin.manager.bbbkey_plugin');
      $virtualEventsCommon = \Drupal::service('virtual_events.common');
      $event_id = $node->bundle() . '_' . $node->uuid();
  
  
      $event = $virtualEventsCommon->getEventById($event_id);
  
      if ($event) {
        // Check if meeting is not active,
        // recreate it before showing the join url
        $event = $event->reCreate();
  
        $entity = $event->getEntity();
        $enabled_event_source = $event->getEnabledSourceKey();
        
        $event_config = $event->getVirtualEventsConfig($enabled_event_source);
        $source_config = $event_config->getSourceConfig($enabled_event_source);
       
        $source_data = $event->getSourceData();
        $eventSourcePlugin = $event->getEventSourcePlugin();
        if(!isset($source_config["data"]["key_type"])) {
          $error_message = $this->t("Couldn't create meeting! please contact system administrator.");      
        }
  
        $keyPlugin = $BBBKeyPluginManager->createInstance($source_config["data"]["key_type"]);
        $keys = $keyPlugin->getKeys($source_config);
  
        $apiUrl = $keys["url"];
        $secretKey = $keys["secretKey"];
        $bbb = new VirtualEventBBB($secretKey, $apiUrl);
  
        // Check access for current entity, if user can update
        // then we can consider the user as moderator,
        // otherwise we consider the user as normal attendee.
        $joinMeetingParams = new JoinMeetingParameters($event->id(), $name, $source_data["settings"]["attendeePW"]);        

        try {
          $joinMeetingParams->setRedirect(TRUE);
          $joinMeetingUrl = $bbb->getJoinMeetingURL($joinMeetingParams);
          $form_state->setResponse(new TrustedRedirectResponse($joinMeetingUrl));
        }
        catch (\RuntimeException $exception) {
          $this->getLogger('social_event_invite_flow')->warning($exception->getMessage());
          $error_message = $this->t("Couldn't get meeting join link! please contact system administrator.");
          $this->messenger()->addError($error_message);
        }
        catch (Exception $exception) {
          $this->getLogger('social_event_invite_flow')->warning($exception->getMessage());
          $error_message = $this->t("Couldn't get meeting join link! please contact system administrator.");
          $this->messenger()->addError($error_message);
        }

      }
    }
  }
}
